using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace MEdiaSpaceCore
{
    public partial class ControlPanel : Form
    {
        public ControlPanel()
        {
            InitializeComponent();
        }

        private void ControlPanel_Load(object sender, EventArgs e)
        {
            // Load 'em up
            Properties.Settings settings = Properties.Settings.Default;
            doorbellPortCombo.SelectedIndex = settings.doorbellport;
            doorMotorDelayField.Text = settings.doormotordelay.ToString();
            doorMotorPortCombo.SelectedIndex = settings.doormotorport;
            homeCompressionSlider.Value = settings.homecompression;
            homeConnectOnStartupButton.Checked = settings.homeconnectonstartup;
            homeDoorbellFlashButton.Checked = settings.homedoorbellflash;
            homeDoorbellFlashDelayField.Text = settings.homedoorbellflashdelay.ToString();
            homeDoorbellFocusButton.Checked = settings.homedoorbellfocus;
            homeDoorbellSoundButton.Checked = settings.homedoorbellsound;
            homeDoorbellSoundField.Text = settings.homedoorbellsoundfile;
            homeFrameRateFastField.Text = settings.homeframeratefast.ToString();
            homeFrameRateSlowField.Text = settings.homeframerateslow.ToString();
            homeMotionSensorFlashButton.Checked = settings.homemotionsensorflash;
            homeMotionSensorFlashDelayField.Text = settings.homemotionsensorflashdelay.ToString();
            homeMotionSensorFocusButton.Checked = settings.homemotionsensorfocus;
            homeMotionSensorSoundButton.Checked = settings.homemotionsensorsound;
            homeMotionSensorSoundField.Text = settings.homemotionsensorsoundfile;
            homeRawRadio.Checked = (settings.homeprotocol == (int)Constants.VIDEO_PROTOCOLS.RAW);
            // homeJPEGRadio is set implicitly
            interfaceKitFirstRadio.Checked = (settings.interfacekitfirst == true);
            // interfaceKitSpecificRadio is set implicitly
            interfaceKitSerialField.Text = settings.interfacekitserial.ToString();
            motionSensorPortCombo.SelectedIndex = settings.motionsensorport;
            motionSensorSensitivityCombo.SelectedIndex = settings.motionsensorsensitivity;
            officeCompressionSlider.Value = settings.officecompression;
            officeFrameRateFastField.Text = settings.officeframeratefast.ToString();
            officeFrameRateSlowField.Text = settings.officeframerateslow.ToString();
            officeRawRadio.Checked = (settings.officeprotocol == (int)Constants.VIDEO_PROTOCOLS.RAW);
            // officeJPEGRadio is set implicitly

            // And then refresh the rest of the UI based on the loaded values
            homeMotionSensorFlashButton_CheckedChanged(null, null);
            homeMotionSensorSoundButton_CheckedChanged(null, null);
            homeDoorbellFlashButton_CheckedChanged(null, null);
            homeDoorbellSoundButton_CheckedChanged(null, null);
            interfaceKit_RadioChanged(null, null);
            homeEncoding_RadioChanged(null, null);
            homeCompression_Scroll(null, null);
            officeEncoding_RadioChanged(null, null);
            officeCompression_Scroll(null, null);
        }

        private void homeMotionSensorFlashButton_CheckedChanged(object sender, EventArgs e)
        {
            homeMotionSensorFlashDelayField.Enabled = homeMotionSensorFlashButton.Checked;
        }

        private void homeMotionSensorSoundButton_CheckedChanged(object sender, EventArgs e)
        {
            homeMotionSensorSoundField.Enabled = homeMotionSensorSoundButton.Checked;
            homeMotionSensorSoundSelectButton.Enabled = homeMotionSensorSoundButton.Checked;
        }

        private void homeMotionSensorSoundSelectButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Sound Files (*.wav)|*.WAV|" +
                         "All Files (*.*)|*.*";
            dlg.CheckFileExists = true;
            dlg.InitialDirectory = Application.StartupPath;

            if (dlg.ShowDialog() == DialogResult.OK)
                homeMotionSensorSoundField.Text = dlg.FileName;
        }

        private void homeDoorbellFlashButton_CheckedChanged(object sender, EventArgs e)
        {
            homeDoorbellFlashDelayField.Enabled = homeDoorbellFlashButton.Checked;
        }

        private void homeDoorbellSoundButton_CheckedChanged(object sender, EventArgs e)
        {
            homeDoorbellSoundField.Enabled = homeDoorbellSoundButton.Checked;
            homeDoorbellSoundSelectButton.Enabled = homeDoorbellSoundButton.Checked;
        }

        private void homeDoorbellSoundSelectButton_Click(object sender, EventArgs e)
        {
            OpenFileDialog dlg = new OpenFileDialog();
            dlg.Filter = "Sound Files (*.wav)|*.WAV|" +
                         "All Files (*.*)|*.*";
            dlg.CheckFileExists = true;
            dlg.InitialDirectory = Application.StartupPath;

            if (dlg.ShowDialog() == DialogResult.OK)
                homeDoorbellSoundField.Text = dlg.FileName;
        }

        private void interfaceKit_RadioChanged(object sender, EventArgs e)
        {
            interfaceKitSerialField.Enabled = interfaceKitSpecificRadio.Checked;
        }

        private void homeEncoding_RadioChanged(object sender, EventArgs e)
        {
            homeCompressionLabel.Enabled = homeJPEGRadio.Checked;
            homeCompressionSlider.Enabled = homeJPEGRadio.Checked;
            homeCompressionPercent.Enabled = homeJPEGRadio.Checked;
        }

        private void homeCompression_Scroll(object sender, EventArgs e)
        {
            homeCompressionPercent.Text = homeCompressionSlider.Value + "%";
        }

        private void officeEncoding_RadioChanged(object sender, EventArgs e)
        {
            officeCompressionLabel.Enabled = officeJPEGRadio.Checked;
            officeCompressionSlider.Enabled = officeJPEGRadio.Checked;
            officeCompressionPercent.Enabled = officeJPEGRadio.Checked;
        }

        private void officeCompression_Scroll(object sender, EventArgs e)
        {
            officeCompressionPercent.Text = officeCompressionSlider.Value + "%";
        }

        private void decimalField_KeyPress(object sender, KeyPressEventArgs e)
        {
            // Get the decimal separator character on this platform
            // (typically a "." for US-English).
            string decimalString = Thread.CurrentThread.CurrentCulture.NumberFormat.CurrencyDecimalSeparator;
            char decimalChar = Convert.ToChar(decimalString);

            if (Char.IsDigit(e.KeyChar) || Char.IsControl(e.KeyChar)) { }
            else if (e.KeyChar == decimalChar && ((TextBox)sender).Text.IndexOf(decimalChar) == -1) { }
            else
            {
                e.Handled = true;
            }
        }

        private void numericField_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (Char.IsDigit(e.KeyChar) || Char.IsControl(e.KeyChar)) { }
            else
            {
                e.Handled = true;
            }
        }
        
        private void acceptButton_Click(object sender, EventArgs e)
        {
            bool validated = false;

            // Commit any editing that might've been going on
            acceptButton.Focus();

            // Validate any open-ended variables
            // TEST CODE
            validated = true;
            // END TEST CODE

            if (validated == true)
            {
                // Write 'em out
                Properties.Settings settings = Properties.Settings.Default;
                settings.doorbellport = doorbellPortCombo.SelectedIndex;
                settings.doormotordelay = Convert.ToDouble(doorMotorDelayField.Text);
                settings.doormotorport = doorMotorPortCombo.SelectedIndex;
                settings.homecompression = homeCompressionSlider.Value;
                settings.homeconnectonstartup = homeConnectOnStartupButton.Checked;
                settings.homedoorbellflash = homeDoorbellFlashButton.Checked;
                settings.homedoorbellflashdelay = Convert.ToDouble(homeDoorbellFlashDelayField.Text);
                settings.homedoorbellfocus = homeDoorbellFocusButton.Checked;
                settings.homedoorbellsound = homeDoorbellSoundButton.Checked;
                settings.homedoorbellsoundfile = homeDoorbellSoundField.Text;
                settings.homeframeratefast = Convert.ToDouble(homeFrameRateFastField.Text);
                settings.homeframerateslow = Convert.ToDouble(homeFrameRateSlowField.Text);
                settings.homemotionsensorflash = homeMotionSensorFlashButton.Checked;
                settings.homemotionsensorflashdelay = Convert.ToDouble(homeMotionSensorFlashDelayField.Text);
                settings.homemotionsensorfocus = homeMotionSensorFocusButton.Checked;
                settings.homemotionsensorsound = homeMotionSensorSoundButton.Checked;
                settings.homemotionsensorsoundfile = homeMotionSensorSoundField.Text;
                if (homeRawRadio.Checked == true)
                    settings.homeprotocol = (int)Constants.VIDEO_PROTOCOLS.RAW;
                else
                    settings.homeprotocol = (int)Constants.VIDEO_PROTOCOLS.JPEG;
                settings.interfacekitfirst = interfaceKitFirstRadio.Checked;
                settings.interfacekitserial = Convert.ToInt32(interfaceKitSerialField.Text);
                settings.motionsensorport = motionSensorPortCombo.SelectedIndex;
                settings.motionsensorsensitivity = motionSensorSensitivityCombo.SelectedIndex;
                settings.officecompression = officeCompressionSlider.Value;
                settings.officeframeratefast = Convert.ToDouble(officeFrameRateFastField.Text);
                settings.officeframerateslow = Convert.ToDouble(officeFrameRateSlowField.Text);
                if (officeRawRadio.Checked == true)
                    settings.officeprotocol = (int)Constants.VIDEO_PROTOCOLS.RAW;
                else
                    settings.officeprotocol = (int)Constants.VIDEO_PROTOCOLS.JPEG;
                settings.Save();

                DialogResult = DialogResult.OK;
            }
            else
                DialogResult = DialogResult.None;
        }
    }
}