using System;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace MEdiaSpaceCore
{
    public partial class CustomForm : Form
    {
        //
        // Thread-safe UI-updating delegates
        //
        protected delegate void SetPictureBoxImage(PictureBox pbox, Image image);
        protected delegate void SetControlText(Control control, String text);
        protected delegate void SetControlEnbabled(Control control, bool enabled);
        protected delegate void SetControlFocus(Control control);


        //
        // Thread-safe UI updaters
        //
        protected void DisplayImageInPictureBox(PictureBox pbox, Image image)
        {
            if (pbox.InvokeRequired)
            {
                SetPictureBoxImage theDelegate = new SetPictureBoxImage(DisplayImageInPictureBox);
                BeginInvoke(theDelegate, new object[] { pbox, image });
            }
            else
            {
                // Handle the boundary cases first
                if (pbox == null || pbox.Height == 0)
                    return;
                if (image == null || image.Height == 0)
                {
                    pbox.Image = null;
                    return;
                }

                // Re-scale the content image to best match the picture box's aspect ratio
                double pboxAR = (double)pbox.Width / (double)pbox.Height;
                double imageAR = (double)image.Width / (double)image.Height;

                int scaledWidth, scaledHeight;
                if (imageAR < pboxAR)
                {
                    scaledWidth = (int)(imageAR * (double)pbox.Height);
                    scaledHeight = pbox.Height;
                }
                else
                {
                    scaledWidth = pbox.Width;
                    scaledHeight = (int)((double)pbox.Width / imageAR);
                }

                Image scaledImage = new Bitmap(scaledWidth, scaledHeight);
                Graphics g = Graphics.FromImage(scaledImage);
                g.DrawImage(image,
                            new Rectangle(0, 0, scaledWidth, scaledHeight),
                            new Rectangle(0, 0, image.Width, image.Height),
                            GraphicsUnit.Pixel);
                g.Dispose();

                pbox.Image = scaledImage;
            }
        }

        protected void DisplayTextInControl(Control control, String text)
        {
            if (control.InvokeRequired)
            {
                SetControlText theDelegate = new SetControlText(DisplayTextInControl);
                BeginInvoke(theDelegate, new object[] { control, text });
            }
            else
            {
                control.Text = text;

                if (control is TextBox && ((TextBox)control).Multiline == true)
                {
                    ((TextBox)control).Select(control.Text.Length, 0);
                    ((TextBox)control).ScrollToCaret();
                }
            }
        }

        protected void EnableControl(Control control, bool enabled)
        {
            if (control.InvokeRequired)
            {
                SetControlEnbabled theDelegate = new SetControlEnbabled(EnableControl);
                BeginInvoke(theDelegate, new object[] { control, enabled });
            }
            else
                control.Enabled = enabled;
        }

        protected void FocusOnControl(Control control)
        {
            if (control.InvokeRequired)
            {
                SetControlFocus theDelegate = new SetControlFocus(FocusOnControl);
                BeginInvoke(theDelegate, new object[] { control });
            }
            else
                control.Focus();
        }
    }
}
